package com.diagbot.aggregate;

import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.facade.BillFacade;
import com.diagbot.facade.CriticalFacade;
import com.diagbot.facade.HighRiskFacade;
import com.diagbot.util.CoreUtil;
import com.diagbot.vo.IndicationPushVO;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @Description: 提醒类聚合
 * @author: zhoutg
 * @time: 2019/10/16 13:37
 */
@Component
@Slf4j
public class IndicationAggregate {

    @Autowired
    BillFacade billFacade;
    @Autowired
    HighRiskFacade highRiskFacade;
    @Autowired
    CriticalFacade criticalFacade;

    @DataProvider("indicationAll")
    public IndicationDTO indicationAll(
            @InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
            @InvokeParameter("indicationPushVO") IndicationPushVO indicationPushVO,
            @InvokeParameter("debug") Map<String, Object> debug,
            @DataConsumer("bill") IndicationDTO billDTO,
            @DataConsumer("highrisk") IndicationDTO highriskDTO,
            @DataConsumer("critical") IndicationDTO criticalDTO) {
        IndicationDTO res = new IndicationDTO();
        if (billDTO != null) {
            res.setBillMsgList(billDTO.getBillMsgList());
            debug.putAll(billDTO.getDebug()); // 调试信息
        }
        if (highriskDTO != null) {
            res.setHighRiskList(highriskDTO.getHighRiskList());
            debug.putAll(highriskDTO.getDebug()); // 调试信息
        }
        if (criticalDTO != null) {
            res.setCriticalValList(criticalDTO.getCriticalValList());
            debug.putAll(criticalDTO.getDebug()); // 调试信息
        }
        res.setDebug(debug);
        return res;
    }

    /**
     * 开单合理性
     *
     * @param wordCrfDTO
     * @param indicationPushVO
     * @return
     */
    @DataProvider("bill")
    public IndicationDTO bill(@InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
                              @InvokeParameter("indicationPushVO") IndicationPushVO indicationPushVO) {
        long start = System.currentTimeMillis();
        IndicationDTO res = new IndicationDTO();
        // 开单合理性
        if (indicationPushVO != null && indicationPushVO.getRuleTypeList().contains("2")) {
            try {
                billFacade.billFac(indicationPushVO, wordCrfDTO, res);
                CoreUtil.getDebugStr(start, "开单规则耗时", res.getDebug());
            } catch (Exception e) {
                log.error("【开单规则出错】" + e.getMessage());
                CoreUtil.getDebugStr("【开单规则出错】", e.getMessage(), res.getDebug());
            }
            return res;
        }
        return null;
    }

    /**
     * 高风险提示
     *
     * @param wordCrfDTO
     * @param indicationPushVO
     * @return
     */
    @DataProvider("highrisk")
    public IndicationDTO highrisk(@InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
                              @InvokeParameter("indicationPushVO") IndicationPushVO indicationPushVO) {
        long start = System.currentTimeMillis();
        IndicationDTO res = new IndicationDTO();
        // 高风险提示
        if (indicationPushVO != null && indicationPushVO.getRuleTypeList().contains("3")) {
            try {
                highRiskFacade.highRiskFac(indicationPushVO, wordCrfDTO, res);
                CoreUtil.getDebugStr(start, "高风险提示耗时", res.getDebug());
            } catch (Exception e) {
                log.error("【高风险提示出错】" + e.getMessage());
                CoreUtil.getDebugStr("【高风险提示出错】", e.getMessage(), res.getDebug());
            }
            return res;
        }
        return null;
    }

    /**
     * 危急值提示
     *
     * @param wordCrfDTO
     * @param indicationPushVO
     * @return
     */
    @DataProvider("critical")
    public IndicationDTO critical(@InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
                                  @InvokeParameter("indicationPushVO") IndicationPushVO indicationPushVO) {
        long start = System.currentTimeMillis();
        IndicationDTO res = new IndicationDTO();
        // 危急值提示
        if (indicationPushVO != null && indicationPushVO.getRuleTypeList().contains("1")) {
            try {
                criticalFacade.criticalFac(indicationPushVO, wordCrfDTO, res);
                CoreUtil.getDebugStr(start, "危急值提示耗时", res.getDebug());
                return res;
            } catch (Exception e) {
                log.error("【危急值提示出错】" + e.getMessage());
                CoreUtil.getDebugStr("【危急值提示出错】", e.getMessage(), res.getDebug());
            }
        }
        return null;
    }
}
