package com.diagbot.facade;

import com.alibaba.fastjson.JSONArray;
import com.diagbot.client.ChiefPresentSimilarityServiceClient;
import com.diagbot.client.StandConvertServiceClient;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.CriticalNeoDTO;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.NeoPushDTO;
import com.diagbot.dto.StandConvertCrfBatchDTO;
import com.diagbot.dto.StandConvertCrfDTO;
import com.diagbot.entity.node.ICDDisease;
import com.diagbot.entity.node.Medicine_Code;
import com.diagbot.entity.node.OralMedicine;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.repository.DiseaseRepository;
import com.diagbot.repository.ICDDiseaseNode;
import com.diagbot.repository.ICDDiseaseRepository;
import com.diagbot.repository.LisNode;
import com.diagbot.repository.LisPackNode;
import com.diagbot.repository.LisPackRepository;
import com.diagbot.repository.LisRepository;
import com.diagbot.repository.MedicineCodeRepository;
import com.diagbot.repository.MedicineNode;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.repository.OperationNode;
import com.diagbot.repository.OperationRepository;
import com.diagbot.repository.PACSNode;
import com.diagbot.repository.PacsRepository;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.BillNeoVO;
import com.diagbot.vo.CriticalNeoVO;
import com.diagbot.vo.HighRiskNeoVO;
import com.diagbot.vo.NeoPushVO;
import com.diagbot.vo.StandConvert;
import com.diagbot.vo.StandConvertCrfVO;
import com.diagbot.vo.neoPushEntity.Diag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 图谱facade
 * @author: Mark
 * @time: 2018/8/6 9:11
 */
@Component
public class NeoFacade {

    @Autowired
    MedicineRepository medicineRepository;
    @Autowired
    PacsRepository pacsRepository;
    @Autowired
    LisRepository lisRepository;
    @Autowired
    LisPackRepository lisPackRepository;
    @Autowired
    DiseaseRepository diseaseRepository;
    @Autowired
    OperationRepository operationRepository;
    @Autowired
    MedicineCodeRepository medicineCodeRepository;
    @Autowired
    ICDDiseaseRepository icdDiseaseRepository;
    @Autowired
    ChiefPresentSimilarityServiceClient chiefPresentSimilarityServiceClient;
    @Autowired
    NeoFacade self;
    @Autowired
    StandConvertServiceClient standConvertServiceClient;
    @Autowired
    RedisUtil redisUtil;
    @Value("${StandConvert.rate}")
    String standConvertRate;

    /**
     * 返回药品缓存信息
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'drugCache'")
    public Map<String, List<String>> getDrugCache() {
        Map<String, List<String>> res = new HashMap<>();

        Iterator<Medicine_Code> iterator = medicineCodeRepository.findAll().iterator();
        Medicine_Code current;
        String med_code, med_type;
        List<OralMedicine> sets;

        while (iterator.hasNext()) {
            current = iterator.next();
            if (current.getMedicine_related_chem().size() > 0) {
                sets = new ArrayList<>(current.getMedicine_related_chem());
                med_code = current.getName();
                med_type = sets.get(0).getName();
                if (res.get(med_type) == null) {
                    res.put(med_type, new ArrayList<>());
                }
                if (res.get(med_type).indexOf(med_code) == -1) {
                    res.get(med_type).add(med_code);
                }
            }
        }

        // TODO 测试数据
        res.put("泰舒达类", Arrays.asList("泰舒达"));
        res.put("青霉素类", Arrays.asList("青霉素"));
        return res;
    }

    /**
     * 图谱反推的数据
     *
     * @param pushVO
     * @return
     */
    public List<NeoPushDTO> getReversePush(NeoPushVO pushVO) {
        List<NeoPushDTO> neoPushDTOS = new ArrayList<>();
        List<String> diags = new ArrayList<>();
        // 如果有诊断名称，则通过诊断反推
        if (null != pushVO.getDiagVo() && pushVO.getDiagVo().getDiags().size() > 0) {
            for (Diag diag : pushVO.getDiagVo().getDiags()) {
                diags.add(diag.getName());
            }

            neoPushDTOS = getDiagInfo(pushVO);
        }

        return neoPushDTOS;
    }


    /**
     * 图谱正推的数据
     *
     * @param pushVO
     * @return
     */
    public List<NeoPushDTO> getPush(NeoPushVO pushVO) {
        List<NeoPushDTO> neoPushDTOS = new ArrayList<>();

        // 如果没有诊断名称，则通过其它信息推送诊断


        return neoPushDTOS;
    }

    /**
     * 处理开单合理性图谱数据
     *
     * @param billNeoVO
     * @return
     */
    public List<BillNeoDTO> getBillNeo(BillNeoVO billNeoVO) {

        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();

        try {
            billNeoDTOs.addAll(getDrugBill(billNeoVO.getDrugList()));
            billNeoDTOs.addAll(getPacsBill(billNeoVO.getPacsList()));
            billNeoDTOs.addAll(getLisDetailBill(billNeoVO.getLisDetailList()));
            billNeoDTOs.addAll(getLisBill(billNeoVO.getLisList()));
            billNeoDTOs.addAll(getOperationBill(billNeoVO.getOperationList()));
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }

        return billNeoDTOs;
    }


    /**
     * 处理高危项目图谱数据
     *
     * @param highRiskNeoVO
     * @return
     */
    public List<HighRiskNeoDTO> getHighRiskNeo(HighRiskNeoVO highRiskNeoVO) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();

        highRiskNeoDTOS.addAll(getHighRiskDrugList(highRiskNeoVO.getDrugList()));
        highRiskNeoDTOS.addAll(getHighRiskOperationList(highRiskNeoVO.getOperationList()));
        return highRiskNeoDTOS;
    }


    /**
     * 获取危急值图谱数据
     */
    public List<CriticalNeoDTO> getCritialNeo(CriticalNeoVO criticalNeoVO) {
        List<CriticalNeoDTO> criticalNeoDTOs = new ArrayList<>();

        criticalNeoDTOs.addAll(getLisCritical(criticalNeoVO));

        return criticalNeoDTOs;
    }


    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(Map<String, String> medsterm) {

        MedicineNode medicineNode = new MedicineNode();
        List<BillNeoDTO> billNeoDTOs = medicineNode.getDrugBill(medsterm, medicineRepository);

        return billNeoDTOs;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm) {

        PACSNode pacsNode = new PACSNode();
        List<BillNeoDTO> billNeoDTOs = pacsNode.getPacsBill(pacsterm, pacsRepository);

        return billNeoDTOs;
    }


    /**
     * 处理化验套餐开单合理性
     */
    public List<BillNeoDTO> getLisBill(Map<String, String> lispackterm) {

        LisPackNode lisPackNode = new LisPackNode();
        List<BillNeoDTO> billNeoDTOs = lisPackNode.getLisBill(lispackterm, lisPackRepository);

        return billNeoDTOs;
    }


    /**
     * 处理化验开单合理性
     */
    public List<BillNeoDTO> getLisDetailBill(Map<String, String> listerm) {

        LisNode lisNode = new LisNode();
        List<BillNeoDTO> billNeoDTOs = lisNode.getLisDetailBill(listerm, lisRepository);

        return billNeoDTOs;
    }



    /**
     * 处理化验开单合理性
     */
    public List<BillNeoDTO> getOperationBill(Map<String, String> listerm) {

        OperationNode opNode = new OperationNode();
        List<BillNeoDTO> billNeoDTOs = opNode.getOperationBill(listerm, operationRepository);

        return billNeoDTOs;
    }


    /**
     * 推送疾病信息
     *
     * @param neoPushVO
     * @return
     */
    public List<NeoPushDTO> getDiagInfo(NeoPushVO neoPushVO) {

        List<NeoPushDTO> neoPushDTOs = new ArrayList<>();
        NeoPushDTO pushDTO;

        ICDDiseaseNode icdDiseaseNode = new ICDDiseaseNode();
        String term;

        for (Diag diag : neoPushVO.getDiagVo().getDiags()) {
            term = diag.getName();

            ICDDisease icdDisease = icdDiseaseRepository.findByNameIs(term).get(0);

            if (icdDisease != null) {
                pushDTO = icdDiseaseNode.ICDDiseasetoPushDTO(icdDisease, neoPushVO);

                if (null != pushDTO) {
                    neoPushDTOs.add(pushDTO);
                }
            }
        }

        return neoPushDTOs;
    }


    /**
     * 获取化验危急值
     */
    public List<CriticalNeoDTO> getLisCritical(CriticalNeoVO criticalNeoVO) {

        LisNode lisNode = new LisNode();
        List<CriticalNeoDTO> criticalNeoDTOs = lisNode.getLisCritialList(criticalNeoVO, lisRepository);

        return criticalNeoDTOs;
    }

    /**
     * 处理高危药品数据
     *
     * @param drugs
     * @return
     */
    public List<HighRiskNeoDTO> getHighRiskDrugList(Map<String, String> drugs) {
        MedicineNode medicineNode = new MedicineNode();
        List<HighRiskNeoDTO> highRiskNeoDTOS = medicineNode.getHighRiskDrug(drugs, medicineRepository);

        return highRiskNeoDTOS;
    }


    /**
     * 处理高危手术和操作数据
     *
     * @param operations
     * @return
     */
    public List<HighRiskNeoDTO> getHighRiskOperationList(Map<String, String> operations) {
        OperationNode operationNode = new OperationNode();
        List<HighRiskNeoDTO> highRiskNeoDTOS = operationNode.getHighRiskOperation(operations, operationRepository);

        return highRiskNeoDTOS;
    }

    /**
     * 提取方法
     *
     * @param jsonArray
     * @param originName
     * @param map
     */
    public void putMap(JSONArray jsonArray, String originName, Map<String, String> map) {
        if (jsonArray.size() == 2) {
            double likeRate = jsonArray.getDoubleValue(1);
            // System.out.println(originName + " " + likeRate);
            if (likeRate > 0.9) {
                String standName = jsonArray.getString(0);
                map.put(originName, standName);
            } else {
                map.put(originName, originName);
            }
        }
    }


    /**
     * 标准词转换
     * 类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 辅助检查：vital"
     *
     * @param standConvert
     * @return Map<String   ,       Map   <   String   ,       String>> -->Map<类型, Map<原始词, 标准词>>
     */
    public Map<String, Map<String, String>> standConvertCrf(StandConvert standConvert) {
        Map<String, Map<String, String>> map = new LinkedHashMap<>();

        List<StandConvertCrfVO> standConvertCrfVOList = new ArrayList<>();
        List<String> clinicalConList = getConvertList(standConvert.getClinicalList(), StandConvertEnum.symptom.getName(), standConvertCrfVOList);
        List<String> operationConList = getConvertList(standConvert.getOperationList(), StandConvertEnum.operation.getName(), standConvertCrfVOList);
        List<String> drugConList = getConvertList(standConvert.getDrugList(), StandConvertEnum.drug.getName(), standConvertCrfVOList);
        List<String> vitallConList = getConvertList(standConvert.getVitalList(), StandConvertEnum.vital.getName(), standConvertCrfVOList);
        List<String> diseaseConList = getConvertList(standConvert.getDiaglList(), StandConvertEnum.disease.getName(), standConvertCrfVOList);
        List<String> pacsConList = getConvertList(standConvert.getPacsList(), StandConvertEnum.pacs.getName(), standConvertCrfVOList);
        List<String> lisConList = getConvertList(standConvert.getLisList(), StandConvertEnum.lis.getName(), standConvertCrfVOList);

        StandConvertCrfBatchDTO standConvertCrfBatchDTO = null;
        try {
            standConvertCrfBatchDTO = standConvertServiceClient.similarityBatch(standConvertCrfVOList);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "标准词转换【服务器】挂了!" + e.getMessage());
        }
        Map<String, Map<String, StandConvertCrfDTO>> crfMap = standConvertCrfBatchDTO.getData();
        getConvertMap(crfMap, StandConvertEnum.symptom.getName(), clinicalConList, standConvert.getClinicalList(), map);
        getConvertMap(crfMap, StandConvertEnum.operation.getName(), operationConList, standConvert.getOperationList(), map);
        getConvertMap(crfMap, StandConvertEnum.drug.getName(), drugConList, standConvert.getDrugList(), map);
        getConvertMap(crfMap, StandConvertEnum.vital.getName(), vitallConList, standConvert.getVitalList(), map);
        getConvertMap(crfMap, StandConvertEnum.disease.getName(), diseaseConList, standConvert.getDiaglList(), map);
        getConvertMap(crfMap, StandConvertEnum.pacs.getName(), pacsConList, standConvert.getPacsList(), map);
        getConvertMap(crfMap, StandConvertEnum.lis.getName(), lisConList, standConvert.getLisList(), map);

        return map;
    }

    /**
     * 入参数据处理
     *
     * @param wordList
     * @param type
     * @param standConvertCrfVOList
     */
    public List<String> getConvertList(List<String> wordList, String type, List<StandConvertCrfVO> standConvertCrfVOList) {
        List<String> convertList = new ArrayList<>();
        if (ListUtil.isEmpty(wordList)) {
            return null;
        }

        // 从redis获取对应的内容
        Map<String, String> redisMap = redisUtil.getValueByType(type);
        if (redisMap == null) {
            redisMap = new LinkedHashMap<>();
        }
        if (ListUtil.isNotEmpty(wordList)) {
            for (String s : wordList) {
                if (StringUtil.isBlank(redisMap.get(s))) {
                    StandConvertCrfVO standConvertCrfVO = new StandConvertCrfVO();
                    standConvertCrfVO.setWord_type(type);
                    standConvertCrfVO.setWord(s);
                    standConvertCrfVOList.add(standConvertCrfVO);
                    if (!convertList.contains(s)) {
                        convertList.add(s);
                    }
                }
            }
        }
        return convertList;
    }

    /**
     * 获取标准词转换map结果，并更新redis
     *
     * @param crfDTO
     * @param type
     * @param convertList
     * @param orginList
     * @param map
     */
    public void getConvertMap(Map<String, Map<String, StandConvertCrfDTO>> crfDTO, String type, List<String> convertList,
                              List<String> orginList, Map<String, Map<String, String>> map) {
        Map<String, StandConvertCrfDTO> crfMap = crfDTO.get(type);
        Map<String, String> typeMap = new LinkedHashMap<>();
        if (ListUtil.isEmpty(convertList) || crfMap == null) {
            for (String orgS : orginList) {
                typeMap.put(orgS, orgS);
            }
        } else {
            // 从redis获取对应的内容
            Map<String, String> redisMap = null;
            redisMap = redisUtil.getValueByType(type);
            if (redisMap == null) {
                redisMap = new LinkedHashMap<>();
            }

            for (String s : convertList) {
                StandConvertCrfDTO standConvertCrfDTO = crfMap.get(s);
                if (standConvertCrfDTO != null) {
                    String rateStr = standConvertCrfDTO.getStandard_words().get(0).get("rate");
                    if (StringUtil.isBlank(rateStr)) {
                        redisMap.put(s, s); // 没有rate就用原词
                    } else {
                        BigDecimal rate = new BigDecimal(rateStr);
                        int flag = rate.compareTo(new BigDecimal(standConvertRate));
                        if (flag < 0) {
                            redisMap.put(s, s); // rate 小于阈值
                        } else {
                            redisMap.put(s, standConvertCrfDTO.getStandard_words().get(0).get("standard_word"));
                        }
                    }
                } else {
                    redisMap.put(s, s); // 找不到就认为是标准词
                }
            }

            // 更新redis
            redisUtil.updateValueByType(redisMap, type);

            for (String orgS : orginList) {
                if (StringUtil.isNotBlank(redisMap.get(orgS))) {
                    typeMap.put(orgS, redisMap.get(orgS));
                } else {
                    typeMap.put(orgS, orgS);
                }
            }
        }
        map.put(type, typeMap);
    }
}
