package com.diagbot.facade;

import com.diagbot.dto.DrugInfoDTO;
import com.diagbot.dto.LisDetailDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.entity.node.ICDDisease;
import com.diagbot.entity.node.LIS;
import com.diagbot.entity.node.LISPack;
import com.diagbot.entity.node.Medicine;
import com.diagbot.entity.node.Operation;
import com.diagbot.entity.node.PACS;
import com.diagbot.repository.DeptNode;
import com.diagbot.repository.DeptRepository;
import com.diagbot.repository.ICDDiseaseNode;
import com.diagbot.repository.ICDDiseaseRepository;
import com.diagbot.repository.LisNode;
import com.diagbot.repository.LisPackNode;
import com.diagbot.repository.LisPackRepository;
import com.diagbot.repository.LisRepository;
import com.diagbot.repository.MedicineNode;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.repository.OperationNode;
import com.diagbot.repository.OperationRepository;
import com.diagbot.repository.PACSNode;
import com.diagbot.repository.PacsRepository;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:44
 */
@Component
public class RetrievalFacade {

    @Autowired
    LisPackRepository lisPackRepository;

    @Autowired
    LisRepository lisRepository;

    @Autowired
    ICDDiseaseRepository icdDiseaseRepository;

    @Autowired
    PacsRepository pacsRepository;

    @Autowired
    MedicineRepository medicineRepository;

    @Autowired
    OperationRepository operationRepository;

    @Autowired
    DeptRepository deptRepository;

    /**
     * 检索
     *
     * @param retrievalVO
     * @return
     */
    public RetrievalDTO index(RetrievalVO retrievalVO) {
        RetrievalDTO retrievalDTO = new RetrievalDTO();

        switch (retrievalVO.getType()) {
            case 1:
                retrievalDTO.setLisNames(getLisPacks(retrievalVO.getInputStr()));
                break;
            case 2:
                retrievalDTO.setLisDetailNames(getLis(retrievalVO.getInputStr()));
                break;
            case 3:
                retrievalDTO.setPacsNames(getPACSs(retrievalVO.getInputStr()));
                break;
            case 4:
                retrievalDTO.setDiseaseNames(getDiseases(retrievalVO.getInputStr()));
                break;
            case 5:
                retrievalDTO.setDrugNames(getMedicines(retrievalVO.getInputStr()));
                break;
            case 6:
                retrievalDTO.setOperationNames(getOperations(retrievalVO.getInputStr()));
                break;
            case 7:
                retrievalDTO.setDeptNames(getDepts(retrievalVO.getInputStr()));
                break;
        }

        return retrievalDTO;
    }


    /*
     * 获取化验套餐名称列表
     */
    public List<String> getLisPacks(String name) {

        LisPackNode lisPackNode = new LisPackNode();
        List<String> lisPacklist = lisPackNode.getLISPackNameList(name, lisPackRepository);

        return lisPacklist;
    }


    /*
     * 获取化验细项名称列表
     */
    public List<LisDetailDTO> getLis(String name) {

        LisNode lisNode = new LisNode();
        List<LisDetailDTO> lislist = lisNode.getLisDetailList(name, lisRepository);

        return lislist;
    }


    /*
     * 获取辅检名称列表
     */
    public List<String> getPACSs(String name) {

        PACSNode pacsNode = new PACSNode();
        List<String> pacslist = pacsNode.getPACSNameList(name, pacsRepository);

        return pacslist;
    }


    /*
     * 获取疾病名称列表
     */
    public List<String> getDiseases(String name) {

        ICDDiseaseNode diseaseNode = new ICDDiseaseNode();
        List<String> diseaselist = diseaseNode.getDiseaseNameList(name, icdDiseaseRepository);

        return diseaselist;
    }


    /*
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicines(String name) {

        MedicineNode medicineNode = new MedicineNode();
        List<DrugInfoDTO> medslist = medicineNode.getMedicineList(name, medicineRepository);

        return medslist;
    }


    /*
     * 获取手术和操作名称列表
     */
    public List<OperationInfoDTO> getOperations(String name) {

        OperationNode operationNode = new OperationNode();
        List<OperationInfoDTO> opslist = operationNode.getOperationList(name, operationRepository);

        return opslist;
    }


    /*
     * 获取科室名称列表
     */
    public List<String> getDepts(String name) {

        DeptNode deptNode = new DeptNode();
        List<String> deptlist = deptNode.getDeptNameList(name, deptRepository);

        return deptlist;
    }

    /**
     * 医学知识（静态信息）检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();
        String inputStr = staticKnowledgeIndexVO.getInputStr();
        List<Integer> types = staticKnowledgeIndexVO.getTypes();
        List<ICDDisease> icdDiseaseList = icdDiseaseRepository.findByNameLike("*" + inputStr + "*");
        List<Medicine> medicineList = medicineRepository.findByNameLike("*" + inputStr + "*");
        List<LISPack> lisPackList = lisPackRepository.findByNameLike("*" + inputStr + "*");
        List<LIS> lisList = lisRepository.findByNameLike("*" + inputStr + "*");
        List<PACS> pacsList = pacsRepository.findByNameLike("*" + inputStr + "*");
        List<Operation> operationList = operationRepository.findByNameLike("*" + inputStr + "*");
        //TODO 诊断
        List<StaticKnowledgeIndexDTO> disRetList = Lists.newLinkedList();
        for (ICDDisease icdDisease : icdDiseaseList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(icdDisease.getName());
            staticDTO.setType(1);
            staticDTO.setTypeName("医保ICD_10疾病");
            disRetList.add(staticDTO);
        }
        //TODO 药品
        List<StaticKnowledgeIndexDTO> drugRetList = Lists.newLinkedList();
        for (Medicine medicine : medicineList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(medicine.getName());
            staticDTO.setType(2);
            staticDTO.setTypeName("药品通用名");
            drugRetList.add(staticDTO);
        }
        //TODO 检验套餐
        List<StaticKnowledgeIndexDTO> lisPackRetList = Lists.newLinkedList();
        for (LISPack lisPack : lisPackList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(lisPack.getName());
            staticDTO.setType(3);
            staticDTO.setTypeName("实验室检查套餐");
            lisPackRetList.add(staticDTO);
        }
        //TODO 检验明细关联出检验套餐
        List<StaticKnowledgeIndexDTO> lisPackRelatecLisRetList = Lists.newLinkedList();
        for (LIS lis : lisList) {
            if (lis.getLisPacks_related_lis().size() > 0) {
                for (LISPack lisPack : lis.getLisPacks_related_lis()) {
                    StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                    staticDTO.setRetrievalName(lis.getName());
                    staticDTO.setName(lisPack.getName());
                    staticDTO.setType(3);
                    staticDTO.setTypeName("实验室检查套餐");
                    lisPackRelatecLisRetList.add(staticDTO);
                }
            }
        }
        //TODO 检验明细
        List<StaticKnowledgeIndexDTO> lisRetList = Lists.newLinkedList();
        for (LIS lis : lisList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(lis.getName());
            staticDTO.setType(4);
            staticDTO.setTypeName("实验室检查");
            lisRetList.add(staticDTO);
        }
        //TODO 检查
        List<StaticKnowledgeIndexDTO> pacsRetList = Lists.newLinkedList();
        for (PACS pacs : pacsList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(pacs.getName());
            staticDTO.setType(5);
            staticDTO.setTypeName("辅助检查");
            pacsRetList.add(staticDTO);
        }
        //TODO 手术和操作
        List<StaticKnowledgeIndexDTO> operationRetList = Lists.newLinkedList();
        for (Operation operation : operationList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(operation.getName());
            staticDTO.setType(6);
            staticDTO.setTypeName("手术和操作");
            operationRetList.add(staticDTO);
        }
        //0-全部、1-诊断、2-药品、3-检验套餐、4-检验明细、5-检查、6-手术和操作
        if (types.contains(0)) {
            retList.addAll(disRetList);
            retList.addAll(drugRetList);
            retList.addAll(lisPackRetList);
            retList.addAll(lisPackRelatecLisRetList);
            retList.addAll(lisRetList);
            retList.addAll(pacsRetList);
            retList.addAll(operationRetList);
        } else {
            if (types.contains(1)) {
                retList.addAll(disRetList);
            }
            if (types.contains(2)) {
                retList.addAll(drugRetList);
            }
            if (types.contains(3)) {
                //检索的套餐
                retList.addAll(lisPackRetList);
                //明细关联出的套餐
                retList.addAll(lisPackRelatecLisRetList);
                //明细
                retList.addAll(lisRetList);
            }
            if (types.contains(5)) {
                retList.addAll(pacsRetList);
            }
            if (types.contains(6)) {
                retList.addAll(operationRetList);
            }
        }
        return retList;
    }
}