package com.diagbot.model.ai.process;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.model.ai.model.EntityEnum;
import com.diagbot.model.ai.model.Lemma;
import com.diagbot.model.entity.*;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class EntityProcessPacs extends EntityProcess {

    public List<Pacs> extractEntity(JSONObject aiOut, Map<String, String> name_uniqueName) {
        //辅检情况
        List<Pacs> pacses = new ArrayList<>();
        List<Lemma> pacsLemmas = createEntityTree(aiOut, EntityEnum.AUXILIARY_EXAMINATION.toString());
        for (Lemma lemma : pacsLemmas) {
            Pacs pacs = new Pacs();
            if(StringUtils.isBlank(lemma.getText().trim())){
                continue;
            }
            pacs.setName(lemma.getText());
            if(name_uniqueName.containsKey(lemma.getText())){
                pacs.setStandName(lemma.getText());
            }
            if (lemma.isHaveChildren()) {
                List<BodyPart> bodyPartList = new ArrayList<>();
                List<Diag> diseaseList = new ArrayList<>();
                for (Lemma relationLemma : lemma.getRelationLemmas()) {
                    if (relationLemma.getProperty().equals(EntityEnum.AUXILIARY_DESCRIPT.toString())) {
                        pacs.setPacsValues(addPacsValue(relationLemma));
                    } else if (relationLemma.getProperty().equals(EntityEnum.BODY.toString())) {
                        bodyPartList.add(addBodyPart(relationLemma));
//                        pacs.setBodyPart(addBodyPart(relationLemma));
                    } else if(relationLemma.getProperty().equals(EntityEnum.DIEASE.toString())){
                        diseaseList.add(addDiag(relationLemma));
                    } else if (relationLemma.getProperty().equals(EntityEnum.OUTERCOURTYARD.toString())) {
                        pacs.setOuterCourtyard(addOuterCourtyard(relationLemma));
                    }
                }
                pacs.setBodyPartList(bodyPartList);
                pacs.setDisease(diseaseList);
            }
            pacses.add(pacs);
        }
        //辅检时间(不是辅检具体项目时间)
        List<Lemma> pdLemmas = createEntityTree(aiOut, EntityEnum.TIME.toString());
        for (Lemma lemma : pdLemmas) {
            PD pd = new PD();
            pd.setName(lemma.getText());
            Pacs pacs = new Pacs();
            pacs.setPd(pd);
            pacses.add(pacs);
        }
        return pacses;
    }

    /**
     * 添加辅助检查描述
     *
     * @param relationLemma
     * @return
     */
    private PacsValue addPacsValue(Lemma relationLemma) {
        PacsValue pacsValue = new PacsValue();
        pacsValue.setName(relationLemma.getText());
        pacsValue.setStandName(relationLemma.getText());
        return pacsValue;
    }

    /**
     * 添加身体部位
     *
     * @param relationLemma
     * @return
     */
    private BodyPart addBodyPart(Lemma relationLemma) {
        BodyPart bodyPart = new BodyPart();
        bodyPart.setName(relationLemma.getText());
        bodyPart.setStandName(relationLemma.getText());
        return bodyPart;
    }

    private Diag addDiag(Lemma relationLemma) {
        Diag bodyPart = new Diag();
        bodyPart.setName(relationLemma.getText());
        bodyPart.setStandName(relationLemma.getText());
        return bodyPart;
    }

    /**
     * 添加外院
     *
     * @param relationLemma
     * @return
     */
    private OuterCourtyard addOuterCourtyard(Lemma relationLemma) {
        OuterCourtyard outerCourtyard = new OuterCourtyard();
        outerCourtyard.setName(relationLemma.getText());
        if (relationLemma.isHaveChildren()) {
            outerCourtyard.setPd(findTime(relationLemma));
            for (Lemma lemma : relationLemma.getRelationLemmas()) {
                if (lemma.getProperty().equals(EntityEnum.AUXILIARY_DESCRIPT.toString())) {
                    outerCourtyard.setPacsValue(addPacsValue(lemma));
                    break;
                }
            }
        }
        return outerCourtyard;
    }

}
