package com.diagbot.model.ai.process;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.model.ai.model.EntityEnum;
import com.diagbot.model.ai.model.Lemma;
import com.diagbot.model.entity.BodyPart;
import com.diagbot.model.entity.Pacs;
import com.diagbot.model.entity.PacsNew;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.util.ListUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @author kwz
 * @date 2020/8/19
 * @time 14:03
 */
public class EntityProcessPacsNew extends EntityProcess {

    public void extractEntity(JSONObject aiOut, PacsLabel pacsLabel) {
        // 获取描述和部位的关系
        Map<String, List<BodyPart>> bodyMap = new LinkedHashMap<>();
        List<Lemma> pacsLemmas = createEntityTree(aiOut, EntityEnum.AUXILIARY_DESCRIPT.toString());
        for (Lemma lemma : pacsLemmas) {
            List<BodyPart> bodyPartList = new ArrayList<>();
            if (lemma.isHaveChildren()) {
                for (Lemma relationLemma : lemma.getRelationLemmas()) {
                    if (relationLemma.getProperty().equals(EntityEnum.BODY.toString())) {
                        BodyPart bodyPart = new BodyPart();
                        bodyPart.setName(relationLemma.getText());
                        bodyPartList.add(bodyPart);
                    }
                }
                bodyMap.put(lemma.getText(), bodyPartList);
            }
        }

        //辅检描述情况
        List<Lemma> pacsAll = createEntityTree(aiOut, EntityEnum.AUXILIARY_EXAMINATION.toString());

        Map<String, PacsNew> map = new HashMap<>();
        for (Lemma lemma : pacsAll) {
            PacsNew pacs = new PacsNew();
            if (lemma.isHaveChildren()) {
                List<Pacs> descList = new ArrayList<>();
                List<Pacs> disList = new ArrayList<>();
                for (Lemma relationLemma : lemma.getRelationLemmas()) {
                    if (relationLemma.getProperty().equals(EntityEnum.AUXILIARY_DESCRIPT.toString())) {
                        Pacs desc = new Pacs();
                        desc.setName(relationLemma.getText());
                        desc.setStandName(relationLemma.getText());
                        if (ListUtil.isNotEmpty(bodyMap.get(relationLemma.getText()))) {
                            desc.setBodyPartList(bodyMap.get(relationLemma.getText()));
                        }
                        descList.add(desc);
                    } else if (relationLemma.getProperty().equals(EntityEnum.DIEASE.toString())) {
                        // 辅检诊断信息
                        Pacs dis = new Pacs();
                        dis.setName(relationLemma.getText());
                        dis.setStandName(relationLemma.getText());
                        disList.add(dis);
                    }
                }
                pacs.setPacsResults(descList);
                pacs.setDisease(disList);
            }
            map.put(lemma.getText(), pacs);
        }
        List<PacsNew> pacsNewList = pacsLabel.getPacsNewList();
        for (PacsNew p : pacsNewList) {
            if (map.get(p.getName()) != null) {
                p.setPacsResults(map.get(p.getName()).getPacsResults());
                p.setDisease(map.get(p.getName()).getDisease());
            }
        }
    }

    /**
     * 添加身体部位
     *
     * @param relationLemma
     * @return
     */
    private BodyPart addBodyPart(Lemma relationLemma) {
        BodyPart bodyPart = new BodyPart();
        bodyPart.setName(relationLemma.getText());
        bodyPart.setStandName(relationLemma.getText());
        return bodyPart;
    }
}
