package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.NeoEnum;
import com.diagbot.facade.NeoFacade;
import com.diagbot.model.entity.Clinical;
import com.diagbot.model.entity.GeneralDesc;
import com.diagbot.model.entity.Medicine;
import com.diagbot.model.entity.Operation;
import com.diagbot.model.entity.Vital;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.DiagLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.DrugRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.rule.SexRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description: 开单总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillProcess {

    @Autowired
    NeoFacade neoFacade;

    public void process(List<BillNeoDTO> billNeoDTOList, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 合并图谱数据到同一个对象中
        List<BillNeoMaxDTO> billNeoMaxDTOList = new ArrayList<>();
        for (BillNeoDTO billNeoDTO : billNeoDTOList) {
            BillNeoMaxDTO billNeoMaxDTO =  new BillNeoMaxDTO();
            if (billNeoDTO.getDrugBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getDrugBillNeoDTO(), billNeoMaxDTO);
            } else if (billNeoDTO.getLisBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getLisBillNeoDTO(), billNeoMaxDTO);
            } else if (billNeoDTO.getPacsBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getPacsBillNeoDTO(), billNeoMaxDTO);
            } else if (billNeoDTO.getOperationBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getOperationBillNeoDTO(), billNeoMaxDTO);
            }
            billNeoMaxDTO.setOrderName(billNeoDTO.getName()); // 开单名称
            billNeoMaxDTO.setOrderStandName(billNeoDTO.getStandname()); // 开单标准名称
            // TODO 测试数据开始
            // if (billNeoDTO.getName().equals("普通胃镜检查")) {
            //     NodeNeoDTO sexNeo = new NodeNeoDTO();
            //     sexNeo.setName("男性");
            //     billNeoMaxDTO.setGender(sexNeo);
            //
            //     NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            //     nodeNeoDTO.setName("红细胞压积");
            //     nodeNeoDTO.setVal("阳性");
            //     billNeoDTO.getPacsBillNeoDTO().getLis().add(nodeNeoDTO);
            //
            //     NodeNeoDTO nodepacsDTO = new NodeNeoDTO();
            //     nodepacsDTO.setName("上腹部平扫");
            //     billNeoDTO.getPacsBillNeoDTO().getPacs().add(nodepacsDTO);
            //
            //     NodeNeoDTO opereat = new NodeNeoDTO();
            //     opereat.setName("胸腔镜下左下肺叶切除术");
            //     billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat);
            //     NodeNeoDTO opereat2 = new NodeNeoDTO();
            //     opereat2.setName("区域淋巴结清扫术");
            //     billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat2);
            //
            //     NodeNeoDTO drug = new NodeNeoDTO();
            //     drug.setName("青霉素类");
            //     billNeoDTO.getPacsBillNeoDTO().getAllergicmeds().add(drug);
            //
            //     NodeNeoDTO drug3 = new NodeNeoDTO();
            //     drug3.setName("泰舒达类");
            //     billNeoDTO.getPacsBillNeoDTO().getOralmeds().add(drug3);
            //
            //     // NodeNeoDTO vital = new NodeNeoDTO();
            //     // vital.setName("体温");
            //     // vital.setMax(new BigDecimal("39.1"));
            //     // vital.setMin(new BigDecimal("35.1"));
            //     // vital.setValType(1);
            //     // billNeoDTO.getPacsBillNeoDTO().getVitals().add(vital);
            //
            //     NodeNeoDTO vitalBp = new NodeNeoDTO();
            //     vitalBp.setName("舒张压");
            //     vitalBp.setMax(new BigDecimal("120"));
            //     vitalBp.setMin(new BigDecimal("100"));
            //     vitalBp.setValType(1);
            //     billNeoDTO.getPacsBillNeoDTO().getVitals().add(vitalBp);
            //
            //     NodeNeoDTO vitalBp2 = new NodeNeoDTO();
            //     vitalBp2.setName("收缩压");
            //     vitalBp2.setMax(new BigDecimal("120"));
            //     vitalBp2.setMin(new BigDecimal("100"));
            //     vitalBp2.setValType(1);
            //     billNeoDTO.getPacsBillNeoDTO().getVitals().add(vitalBp2);
            // }
            // 测试数据结束

            System.out.println(billNeoMaxDTO);
            billNeoMaxDTOList.add(billNeoMaxDTO);
        }
        processRule(billNeoMaxDTOList, wordCrfDTO, res);
    }

    // 规则处理
    public void processRule(List<BillNeoMaxDTO> billNeoMaxDTOList, WordCrfDTO wordCrfDTO, IndicationDTO indicationDTO) {
        // 获取药品缓存
        Map<String, List<String>> drugMap = neoFacade.getDrugCache();
        DiagLabel diagLabel = wordCrfDTO.getDiagLabel();
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();

        // 体征
        List<Vital> vitals = wordCrfDTO.getVitalLabel().getVitals();

        // 诊断数据
        List<Item> diags = wordCrfDTO.getDiag();

        // 手术数据
        List<Operation> operations_present = presentLabel.getOperations();
        List<Operation> operations_past = pastLabel.getOperations();
        operations_present.addAll(operations_past);

        // 临床表现数据
        List<Clinical> clinicals = new ArrayList<>();
        if(chiefLabel != null && ListUtil.isNotEmpty(chiefLabel.getClinicals())){
            clinicals.addAll(chiefLabel.getClinicals());
        }
        // 现病史临床表现数据先不用
        // if(presentLabel != null && ListUtil.isNotEmpty(presentLabel.getClinicals())){
        //     clinicals.addAll(presentLabel.getClinicals());
        // }

        List<BillMsg> billMsgList = new ArrayList<>();
        for (BillNeoMaxDTO bill : billNeoMaxDTOList) {
            // 性别
            SexRule.compareSexWithBill(wordCrfDTO, bill, billMsgList, NeoEnum.gender.getName());

            // 年龄
            AgeRule.compareAgeWithBill(wordCrfDTO, bill, billMsgList, NeoEnum.ageNeoDTO.getName());

            // 诊断
            CommonRule.compareItemWithBill(bill.getDisease(), diags, bill, billMsgList, NeoEnum.disease.getName());

            // 化验
            LisRule.compareLisWithBill(wordCrfDTO.getLis(), bill, billMsgList, NeoEnum.lis.getName());

            // 体征
            VitalRule.compareVitalWithBill(vitals, bill, billMsgList, NeoEnum.vitals.getName());

            // 辅检
            PacsRule.comparePacsWithBill(bill.getPacs(), wordCrfDTO.getPacs(), bill, billMsgList, NeoEnum.pacs.getName());

            // 临床表现
            CommonRule.compareNameWithBill(bill.getClinicfindings(), clinicals, bill, billMsgList, NeoEnum.clinicfindings.getName());

            // 手术(既往史、现病史)
            CommonRule.compareNameWithBill(bill.getOperations(), operations_present, bill, billMsgList, NeoEnum.operations.getName());

            // 禁忌过敏药品（既往史）
            DrugRule.compareDrugWithBill(bill.getAllergicmeds(), pastLabel.getAllergyMedicines(), bill, billMsgList, drugMap, NeoEnum.allergicmeds.getName());

            // 服用药品（现病史一般情况后的药品）
            DrugRule.compareDrugWithBill(bill.getOralmeds(), takeMedicine(presentLabel), bill, billMsgList, drugMap, NeoEnum.oralmeds.getName());

            // 禁用人群
            // GroupRule.compareGroupWithBill(wordCrfDTO, bill, billMsgList, NeoEnum.group.getName());
        }

        // 24小时重复开单项
        CommonRule.repeat24Bill(wordCrfDTO, billMsgList);
        indicationDTO.setBillMsgList(billMsgList);
    }

    /**
     * 一般情况以后的药品
     * @param presentLabel
     */
    public List<Medicine> takeMedicine(PresentLabel presentLabel){
        List<Medicine> takems = new ArrayList<>();
        List<GeneralDesc> generals = presentLabel.getGenerals();
        List<Medicine> medicines = presentLabel.getMedicines();
        if (generals.size() > 0) {
            String presentText = presentLabel.getText();
            if (StringUtils.isNotBlank(presentText) && medicines != null && medicines.size() > 0) {
                String lastGeneral = generals.get(generals.size() - 1).getName();
                int lastGeneralIndex = presentText.lastIndexOf(lastGeneral);
                for (Medicine medicine : medicines) {
                    /* 现病史中一般情况之后的药品名称,并且不包含不详 */
                    int medicine_index = presentText.indexOf(medicine.getName());
                    if (medicine_index > lastGeneralIndex && !medicine.getName().contains("不详")) {
                        takems.add(medicine);
                    }
                }
            }
        }
        return takems;
    }
}
