package com.diagbot.repository;

import com.diagbot.dto.*;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MedicineNode {


    public DrugBillNeoDTO MedicinetoDrugDTO(Medicine medicine) {
        DrugBillNeoDTO drugBillNeoDTO = new DrugBillNeoDTO();

        Age age = medicine.getAge();
        if (age!=null) {
            NodeNeoDTO ageNeoDTO = new NodeNeoDTO();
            ageNeoDTO.setMin(BigDecimal.valueOf(age.getMinval()));
            ageNeoDTO.setMax(BigDecimal.valueOf(age.getMaxval()));
            drugBillNeoDTO.setAgeNeoDTO(ageNeoDTO);
        }


        Gender gender = medicine.getGender();
        if (gender!=null) {
            drugBillNeoDTO.setGender(NeoUtil.updateNodeInfo(gender.getName(), null, null, null));
        }


        Set<ClinicalFinding> clinicalfindings = medicine.getClinicalfindings();
        for (ClinicalFinding clinicalfinding : clinicalfindings) {
            String name = clinicalfinding.getName();
            BigDecimal min = NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval());
            BigDecimal max = NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval());
            String unit = clinicalfinding.getUnit();
            drugBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(name, min, max, unit));
        }


        Set<Vital> vitals = medicine.getVitals();
        for (Vital vital : vitals) {
            String name = vital.getName();
            BigDecimal min = NeoUtil.DoubletoBigDecimal(vital.getMinval());
            BigDecimal max = NeoUtil.DoubletoBigDecimal(vital.getMaxval());
            String unit = vital.getUnit();
            drugBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(name, min, max, unit));
        }


        Set<LIS> lisset = medicine.getLisset();
        for (LIS lis : lisset) {
            String name = lis.getName();
            BigDecimal min = NeoUtil.DoubletoBigDecimal(lis.getMinval());
            BigDecimal max = NeoUtil.DoubletoBigDecimal(lis.getMaxval());
            String unit = lis.getUnit();
            drugBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(name, min, max, unit));
        }


        Set<PACS> pacsset = medicine.getPacs();
        for (PACS pacs : pacsset) {
            String name = pacs.getName();
//            BigDecimal min = pacs.getMinval();
//            BigDecimal max = pacs.getMaxval();
//            String unit = pacs.getUnit();
//            drugBillNeoDTO.getPacs().add(NeoUtil.updateNodeInfo(name, min, max, unit));
            drugBillNeoDTO.getPacs().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        Set<AllergicMed> allergicmeds = medicine.getAllergicmeds();
        for (AllergicMed allergicmed: allergicmeds) {
            String name = allergicmed.getName();
            drugBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        Set<Operation> operations = medicine.getOperations();
        for (Operation operation : operations) {
            String name = operation.getName();
            drugBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        Set<Procedure> procedures = medicine.getProcedures();
        for (Procedure procedure : procedures) {
            String name = procedure.getName();
            drugBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        Set<Disease> diseases = medicine.getDiseases();
        for (Disease disease : diseases) {
            String name = disease.getName();
            drugBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        Set<Group> groups = medicine.getGroups();
        for (Group group : groups) {
            String name = group.getName();
            drugBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        Set<ConflictMed> conflict_medicines = medicine.getConflictmeds();
        for (ConflictMed conflict_medicine : conflict_medicines) {
            String name = conflict_medicine.getName();
            drugBillNeoDTO.getConflictmeds().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        return drugBillNeoDTO;
    }



    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(Map<String, String> medsterm, MedicineRepository medicineRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        DrugBillNeoDTO drugBillNeoDTO;

        Medicine medicine;
        String medname;

        for (String term : medsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            medname = medsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(medname);

            drugBillNeoDTO = new DrugBillNeoDTO();

            medicine = medicineRepository.findByNameIs(medname);

            if (medicine != null) {
                drugBillNeoDTO = MedicinetoDrugDTO(medicine);
            }

            billNeoDTO.setDrugBillNeoDTO(drugBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }


        return billNeoDTOs;
    }



    /**
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicineList(String name, MedicineRepository medicineRepository) {
        String nodename;

        List<DrugInfoDTO> medslist = new ArrayList<>();
        DrugInfoDTO drugInfoDTO;

        List<Medicine> medicines = medicineRepository.findByNameLike("*"+name+"*");

        for (Medicine med : medicines) {
            drugInfoDTO = new DrugInfoDTO();
            nodename = med.getName();

            drugInfoDTO.setName(nodename);
            medslist.add(drugInfoDTO);
        }


        return medslist;
    }



    /**
     * 获取高危药品数据
     */
    public List<HighRiskNeoDTO> getHighRiskDrug(Map<String, String> drugsterm, MedicineRepository medicineRepository) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();
        HighRiskNeoDTO highRiskNeoDTO;
        String drugname;
        Medicine medicine;

        for (String term : drugsterm.keySet()) {
            highRiskNeoDTO = new HighRiskNeoDTO();

            drugname = drugsterm.get(term);
            highRiskNeoDTO.setName(term);
            highRiskNeoDTO.setStandname(drugname);

            medicine = medicineRepository.findByNameIs(drugname);

            if (null != medicine && null!=medicine.getRisklevel() && medicine.getRisklevel().trim().length()>0) {
                highRiskNeoDTO.setIsHighrisk(true);
                highRiskNeoDTO.setType("药品");
                highRiskNeoDTOS.add(highRiskNeoDTO);
            }
        }

        return highRiskNeoDTOS;
    }
}

