package com.diagbot.repository;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.PacsBillNeoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class PACSNode {


    public PacsBillNeoDTO PacstoPACSDTO(PACS pacs) {
        PacsBillNeoDTO pacsBillNeoDTO = new PacsBillNeoDTO();


        Age age = pacs.getAge();
        if (age!=null) {
            NodeNeoDTO ageNeoDTO = new NodeNeoDTO();
            ageNeoDTO.setMin(BigDecimal.valueOf(age.getMinval()));
            ageNeoDTO.setMax(BigDecimal.valueOf(age.getMaxval()));
            pacsBillNeoDTO.setAgeNeoDTO(ageNeoDTO);
        }


        Gender gender = pacs.getGender();
        if (gender!=null) {
            pacsBillNeoDTO.setGender(NeoUtil.updateNodeInfo(gender.getName(), null, null, null));
        }



        Set<ClinicalFinding> clinicalfindings = pacs.getClinicalfindings();
        for (ClinicalFinding clinicalfinding : clinicalfindings) {
            String name = clinicalfinding.getName();
            BigDecimal min = NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval());
            BigDecimal max = NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval());
             String unit = clinicalfinding.getUnit();
            pacsBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(name, min, max, unit));
        }


        Set<Vital> vitals = pacs.getVitals();
        for (Vital vital : vitals) {
            String name = vital.getName();
            BigDecimal min = NeoUtil.DoubletoBigDecimal(vital.getMinval());
            BigDecimal max = NeoUtil.DoubletoBigDecimal(vital.getMaxval());
            String unit = vital.getUnit();
            pacsBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(name, min, max, unit));
        }


        Set<Medicine> conflict_medicines = pacs.getConflict_medicines();
        for (Medicine conflict_medicine : conflict_medicines) {
            String name = conflict_medicine.getName();
            pacsBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }

        Set<AllergicMed> allergicmeds = pacs.getAllergicmeds();
        for (AllergicMed allergicmed : allergicmeds) {
            String name = allergicmed.getName();
            pacsBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }

        Set<Operation> operations = pacs.getOperations();
        for (Operation operation : operations) {
            String name = operation.getName();
            pacsBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }

        Set<Procedure> procedures = pacs.getProcedures();
        for (Procedure procedure : procedures) {
            String name = procedure.getName();
            pacsBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }

        Set<LIS> liss = pacs.getLiss();
        for (LIS lis : liss) {
            String name = lis.getName();
            BigDecimal min = NeoUtil.DoubletoBigDecimal(lis.getMinval());
            BigDecimal max = NeoUtil.DoubletoBigDecimal(lis.getMaxval());
            String unit = lis.getUnit();
            pacsBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(name, min, max, unit));
        }

        Set<Disease> diseases = pacs.getDiseases();
        for (Disease disease : diseases) {
            String name = disease.getName();
            pacsBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }

        Set<Group> groups = pacs.getGroups();
        for (Group group : groups) {
            String name = group.getName();
            pacsBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(name, null, null, null));
        }


        return pacsBillNeoDTO;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm, PacsRepository pacsRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        PacsBillNeoDTO pacsBillNeoDTO;

        List<PACS> pacslt;
        String pacsname;

        for (String term : pacsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            pacsname = pacsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(pacsname);

            pacsBillNeoDTO = new PacsBillNeoDTO();

            pacslt = pacsRepository.findByNameIs(pacsname);

            if (pacslt != null && pacslt.size()>0) {
                pacsBillNeoDTO = PacstoPACSDTO(pacslt.get(0));
            }

            billNeoDTO.setPacsBillNeoDTO(pacsBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    /**
     * 获取辅检名称列表
     */
    public List<String> getPACSNameList(String name, PacsRepository pacsRepository) {
        String pacsname;

        List<String> pacslist = new ArrayList<>();

        List<PACS> pacslt = pacsRepository.findByNameLike("*"+name+"*");

        for (PACS pacs : pacslt) {
            pacsname = pacs.getName();
            if (!pacslist.contains(pacsname)) {
                pacslist.add(pacsname);
            }
        }


        return pacslist;
    }
}






