package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.model.entity.Negative;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 实验室检查和辅助检查规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
public class DrugRule {

    /**
     * 比较药品是否存在(图谱返回的是药品大类，需要从缓存中获取大类对应的药品名称)
     *
     * @param drugLises
     * @param inputLises
     * @return
     */
    public static <T> void compareDrugWithBill(List<NodeNeoDTO> drugLises, List<T> inputLises, BillNeoMaxDTO billNeoMaxDTO,
                                               List<BillMsg> billMsgList, Map<String, List<String>> drugMap, String type) {
        Map<String, String> old_stand = new HashMap<>();
        if (ListUtil.isNotEmpty(drugLises) && ListUtil.isNotEmpty(inputLises)) {
            List<String> drugType = drugLises.stream().map(x -> x.getName()).collect(Collectors.toList());
            List<String> allDrug = new ArrayList<>(); // 所有的药品名称
            for (String drugT : drugType) {
                if (drugMap.get(drugT) != null) {
                    allDrug.addAll(drugMap.get(drugT));
                }
            }
            for (T d : inputLises) {
                Negative val = (Negative) CoreUtil.getFieldValue(d, "negative");
                if (val == null) {
                    String c = (String) CoreUtil.getFieldValue(d, "standName");
                    String c_name = (String) CoreUtil.getFieldValue(d, "name");
                    if (StringUtils.isNotBlank(c) && allDrug.contains(c)) {
                        old_stand.put(c_name, c);
                    }
                }
            }
            if (old_stand.size() > 0) {
                for (Map.Entry<String, String> d : old_stand.entrySet()) {
                    String message = "";
                    if ("禁忌过敏药品".equals(type)) {
                        message = d.getKey() + "过敏";
                    } else {
                        message = "已开" + d.getKey();
                    }
                    BillMsg commonBillMsg = CoreUtil.getCommonBillMsg(
                            billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                            message, type);
                    billMsgList.add(commonBillMsg);
                }
            }
        }
    }
}
